/**
 * DLL functions for Java to C++ interface using Java Native Interface (JNI).
 * See kregdb_KingsrowEgdbJNI.h, which is generated by the 'javah' tool.
 * Function names are JNI-generated, corresponding to:
 * Java, package kregdb, class KingsrowEgdbJNI, native functions
 * egdbOpenLog(), egdbOpen(), egdbLookup(), egdbClose(), egdbCloseLog().
 *
 * Author: Jan-Jaap van Horssen
 * Date  : 2017-03-13
 */

#include <iostream>
#include "egdb/egdb_intl.h"
#include "kregdb_KingsrowEgdbJNI.h"

using namespace std;
using namespace egdb_interface;

/**
 * Instead of passing a Java callback print_msgs() function to egdb_open(), 
 * we use my_print_msgs() and a local file stream.
 * So there can only be one log file open at a time for multiple egdbOpen() calls.
 *
 * Note: If the log file is already open in another process, opening the log file
 * will overwrite the current one.
 */

FILE *my_stream = NULL;

void my_print_msgs(char const *msg) {
	if (!my_stream) {
		cout << msg;	// to Java console (buffered)
		return;
	}
	fprintf(my_stream, "%s", msg);
}

/**
 * Exported functions
 */

JNIEXPORT jint JNICALL Java_kregdb_KingsrowEgdbJNI_egdbOpenLog(JNIEnv *env, jobject thisObj, jstring logfile) {
	if (my_stream) {
		return 1;
	}
	const char *clog = env->GetStringUTFChars(logfile, NULL);
	if ((my_stream = fopen(clog, "w")) == NULL) {
		return 2;
	}
	return 0;
}

JNIEXPORT jlong JNICALL Java_kregdb_KingsrowEgdbJNI_egdbOpen(JNIEnv *env, jobject thisObj, jstring options, jint cache_mb, jstring dir) {
	const char *copt = env->GetStringUTFChars(options, NULL);
	const char *cdir = env->GetStringUTFChars(dir, NULL);
	EGDB_DRIVER *handle = egdb_open(copt, cache_mb, cdir, my_print_msgs);
	return (jlong)(intptr_t)handle;		// 64-bit: (jlong) suffices; 32-bit: needs extra (intptr_t), not tested
}

JNIEXPORT jint JNICALL Java_kregdb_KingsrowEgdbJNI_egdbLookup(JNIEnv *env, jobject thisObj, jlong handle, jlong black, jlong white, jlong king, jint color, jint cl) {
	const EGDB_POSITION pos = { (EGDB_BITBOARD)black, (EGDB_BITBOARD)white, (EGDB_BITBOARD)king };
	return egdb_lookup((EGDB_DRIVER *)handle, &pos, color, cl);
}

JNIEXPORT void JNICALL Java_kregdb_KingsrowEgdbJNI_egdbClose(JNIEnv *env, jobject thisObj, jlong handle) {
	egdb_close((EGDB_DRIVER *)handle);
}

JNIEXPORT void JNICALL Java_kregdb_KingsrowEgdbJNI_egdbCloseLog(JNIEnv *env, jobject thisObj) {
	if (my_stream) {
		fclose(my_stream);
	}
}
